/*
 * The content of this file is a mix of rpcgen-generated
 * and hand-edited program text.  It is not automatically
 * generated by, e.g., build processes.
 *
 * This file is under version control.
 */

#include "config.h"

#include "nfsv41.h"

/* Most Ganesha XDR decoder routines are inlines, defined in
 * nfsv41.h.  The present file is retained as a location for
 * XDR decoder routines with external linkage.
 */
